<?php
/**
 * 2018-2022 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2022 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;

require_once _PS_MODULE_DIR_.'mxb_docnum/classes/MxbDocnumUtils.php';
//require_once _PS_MODULE_DIR_.'mxb_tools/classes/MxbToolsDebug.php';

/** Class class DeliverySlipData */
class MxbDocnumDeliverySlipData extends ObjectModel
{
    public static $className = 'MxbDocnumDeliverySlipData';
    public static $moduleName = 'mxb_docnum';
    /*
        static $fieldList = array(array(  'field'=>'customer_code',  'dimension'=>'required'=>false),
            array(  'field'=>'company',      'required'=>true),
            array(  'field'=>'vat_number',    'required'=>true),
            array(  'field'=>'address1',    'required'=>true),
            array(  'field'=>'address2',    'required'=>false),
            array(  'field'=>'postcode',    'required'=>true),
            array(  'field'=>'city',      'required'=>true),
            array(  'field'=>'id_country',    'required'=>true),
            array(  'field'=>'id_state',    'required'=>true),
            array(  'field'=>'phone',      'required'=>true),
        );
    */

    //public $id;
    //public $id_mxb_docnum_delivery_slip_data;
    public $id_order;
    public $transport_reason;
    public $transport_by;
    public $carriage;
    public $appearance;
    public $package_quantity;
    public $package_weight;
    public $delivery_start_date;
    public $notes;

    public static $definition = array(
        'table' => 'mxb_docnum_delivery_slip_data',
        'primary' => 'id_mxb_docnum_delivery_slip_data',
        'fields' => array(
            'transport_reason' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 32),
            'transport_by' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 32),
            'carriage' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 32),
            'appearance' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 32),
            'package_quantity' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 8),
            'package_weight' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 16),
            'delivery_start_date' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 32),
            'notes' => array('type' => self::TYPE_STRING, 'validate' => 'isAnything', 'required' => false, 'size' => 64),
            'id_order' => array('type' => self::TYPE_INT, 'copy_post' => false),
        ),
    );

    protected $webserviceParameters = array(
        'fields' => array(
            // 'id' => array('sqlId' => 'id_mxb_docnum_special_discount', 'xlink_resource' => 'mxb_docnum_special_discount_lang'),
            //'id_mxb_docnum_decoration_method' => array('xlink_resource' => 'mxb_docnum_decoration_method'),
            //'date_add' => array('sqlId' => 'date_add'),
        ),
    );

    public function update($nullValues = false)
    {
        //MxbToolsDebug::writeLog('mxbmemeberprogram', 'MxbmpCustomerProductRule::update()', 'record id='.$this->id . ', company='.$this->company);
        return parent::update($nullValues);
    }



    public function save($null_values = false, $auto_date = true)
    {
        //MxbToolsDebug::writeLog('mxbmemeberprogram', 'MxbmpCustomerProductRule::save()', 'record id='.$this->id . ', company='.$this->company);
        return parent::save($null_values, $auto_date);
    }

    public function delete()
    {
        //MxbToolsDebug::writeLog('mxbmemeberprogram', 'MxbmpCustomerProductRule::delete()', 'record id='.$this->id . ', company='.$this->company);
        return parent::delete();
    }

    public static function getDeliverySlipData($id_mxb_docnum_delivery_slip_data = 0, $id_order = 0, $limit = 1)
    {
        // public function getArray()
        $sql = '
            SELECT *
            FROM ' . _DB_PREFIX_ . 'mxb_docnum_delivery_slip_data
            WHERE 1';

        if ($id_mxb_docnum_delivery_slip_data > 0) {
            $sql .= ' AND id_mxb_docnum_delivery_slip_data = ' . (int) $id_mxb_docnum_delivery_slip_data;
        }
        if ($id_order > 0) {
            $sql .= ' AND id_order = ' . (int) $id_order;
        }
        $sql .= ' ORDER BY id_order ASC';
        if ((int)$limit > 0) {
            $sql .= ' LIMIT ' . (int)$limit;
        }
        //MxbToolsDebug::writeLog("mxb_docnum", 'MxbmpCustomerAdditionalData::getCustomerAdditionalData', 'sql=' . $sql);
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
    }

    public static function getDeliverySlipDataForSmarty($id_order = 0)
    {
        $data = self::getDeliverySlipData(0, $id_order);
        if (!empty($data)) {
            return $data[0];
        }
        return self::newEmptyDeliverySlipDataArray($id_order);
    }

    public static function newEmptyDeliverySlipDataArray($id_order = 0)
    {
        $data = array();
        foreach (self::$definition['fields'] as $key => $field) {
            switch ($field['type']) {
                case self::TYPE_BOOL:
                    $val = false;
                    break;
                case self::TYPE_STRING:
                    $val = $key == 'delivery_start_date' ? date('Y-m-d H:i:s') : "";
                    break;
                case self::TYPE_DATE:
                    $val = Tools::formatDateStr(date('Y-m-d H:i:s'), true);
                    break;
                //case self::TYPE_INT:
                default:
                    $val = 0;
                    break;
            }
            $data[$key] = $val;
        }
        $data['id_mxb_docnum_delivery_slip_data'] = 0;
        $data['id_order'] = $id_order;
        return $data;
    }

    public static function duplicateDeliverySlipData($id_mxb_docnum_delivery_slip_data = 0, $id_order = 0)
    {
        $data = self::getLastDeliverySlipData($id_mxb_docnum_delivery_slip_data, $id_order);
        if (empty($data)) {
            return self::newEmptyDeliverySlipDataArray();
        }

        // save new
        $sql = "INSERT INTO `"._DB_PREFIX_."mxb_docnum_delivery_slip_data` (";
        // collect fields and values
        $values = '';
        foreach ($data as $key => $value) {
            if ($key != "id_mxb_docnum_delivery_slip_data") {
                $sql .= $key . ', ';
                $values .= "'" . addslashes($value) . "', ";
            }
        }
        // remove last ', ' from strings
        $sql = Tools::substr($sql, 0, -2);
        $values = Tools::substr($values, 0, -2);
        // finish sql
        $sql .= ') VALUES (' . $values . ');';
        $res = Db::getInstance()->execute($sql);
        if ($res) {
            $data['id_mxb_docnum_delivery_slip_data'] = Db::getInstance()->Insert_ID();
        }
        return $data;
    }

    public static function getLastDeliverySlipData($id_mxb_docnum_delivery_slip_data = 0, $id_order = 0)
    {
        $sql = '
            SELECT *
            FROM ' . _DB_PREFIX_ . 'mxb_docnum_delivery_slip_data
            WHERE 1';

        if ($id_mxb_docnum_delivery_slip_data > 0) {
            $sql .= ' AND id_mxb_docnum_delivery_slip_data = ' . (int) $id_mxb_docnum_delivery_slip_data;
        }
        if ($id_order > 0) {
            $sql .= ' AND id_order = ' . (int) $id_order;
        }
        $sql .= ' ORDER BY id_mxb_docnum_delivery_slip_data DESC
            LIMIT 1';

        //MxbToolsDebug::writeLog("mxb_docnum", 'MxbmpCustomerAdditionalData::getCustomerAdditionalData', 'sql=' . $sql);

        $data = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        if (empty($data)) {
            return null;
        }
        //data[0][] = Tools::formatDateStr(date('Y-m-d H:i:s'), true);
        return $data[0];
    }
    
    public static function deliverySlipAddDataIsEnabled()
    {
        return !((bool)Configuration::getGlobalValue('MXB_DOCNUM_DELIVERY_SLIP_ADD_DATA_ENABLED') === false || MxbDocnumUtils::isModuleEnabled('mxbmemberprogram') || MxbDocnumUtils::isModuleEnabled('gadgetcard'));
    }
    public static function deliverySlipCustomTemplateIsEnabled()
    {
        return !((bool)Configuration::getGlobalValue('MXB_DOCNUM_DELIVERY_SLIP_CUSTOM_TEMPLATE_ENABLED') === false || MxbDocnumUtils::isModuleEnabled('mxbmemberprogram') || MxbDocnumUtils::isModuleEnabled('gadgetcard'));
    }
    
    public static function getTrackingNumber($id_order = 0)
    {
        // get tracking if exists
        $sql = '
            SELECT tracking_number
            FROM ' . _DB_PREFIX_ . 'order_carrier
            WHERE id_order = "' . $id_order . '"';

        $ret = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS($sql);
        $tracking = count($ret) ? $ret[0]['tracking_number'] : '';
        return $tracking;
    }

    public static function getTemplateOverride($template_name)
    {
        // MXB_DOCNUM OVERRIDE: modify template folder
        if (!self::deliverySlipCustomTemplateIsEnabled()) {
            return false; //parent::getTemplate($template_name);
        }
        $template_id = (int)Configuration::getGlobalValue('MXB_DOCNUM_DELIVERY_SLIP_CUSTOM_TEMPLATE_ID');
        $mxb_template = rtrim(_PS_MODULE_DIR_, DIRECTORY_SEPARATOR)
            . DIRECTORY_SEPARATOR . 'mxb_docnum'
            . DIRECTORY_SEPARATOR . 'views'
            . DIRECTORY_SEPARATOR . 'templates'
            . DIRECTORY_SEPARATOR . 'hook'
            . DIRECTORY_SEPARATOR . 'pdf'
            . DIRECTORY_SEPARATOR . $template_id
            . DIRECTORY_SEPARATOR . $template_name . '.tpl';

        if (file_exists($mxb_template)) {
            $template = $mxb_template;
        } else {
            $template = false; // parent::getTemplate($template_name);
        }
        return $template;
    }

    public function getHeaderOverride($smarty = null, $shop_address)
    {
        // MXB_DOCNUM OVERRIDE
        if ((bool)Configuration::getGlobalValue('MXB_DOCNUM_DELIVERY_SLIP_ALT_ADDRESS')) {
            $shop_address = str_replace(
                "\n",
                "<br />",
                (string)Configuration::getGlobalValue('MXB_DOCNUM_DELIVERY_SLIP_ALT_ADDRESS')
            );
        }
        if ($smarty) {
            $smarty->assign(array('shop_address' => $shop_address));
            $module = Module::getInstanceByName('mxb_docnum');
            $smarty->assign(array('header_title' => $module->l('DELIVERY SLIP')));
        }
        return $shop_address;
    }

    public function getContentOverride($smarty, $id_order, $template)
    {
        // MXB_DOCNUM OVERRIDE
        if (!self::deliverySlipCustomTemplateIsEnabled()) {
            return false;
        }
        // recover delivery slip data
        if (self::deliverySlipAddDataIsEnabled()) {
            $deliverySlipData = self::getDeliverySlipDataForSmarty($id_order);
            $smarty->assign('deliverySlipData', $deliverySlipData);
            // get tracking if exists
            $smarty->assign('mxb_tracking_number', self::getTrackingNumber($id_order));
        }
        $smarty->assign('delivery_slip_adv_table_tab', $smarty->fetch($template));
    }
}
