<?php
/**
 * 2007-2019 PrestaShop and Contributors
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2019 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 * International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_.'mxb_docnum/classes/MxbDocnumUtils.php';
require_once _PS_MODULE_DIR_.'mxb_docnum/classes/MxbDocnumDeliverySlipData.php';

class HTMLTemplateDeliverySlip extends HTMLTemplateDeliverySlipCore
{

    public function __construct(OrderInvoice $order_invoice, $smarty, $bulk_mode = false)
    {
        parent::__construct($order_invoice, $smarty, $bulk_mode);

        // MXB_DOCNUM OVERRIDE: modify delivery slip number
        $ref = MxbDocnumUtils::generateCustomReference('delivery-slip', (int)$this->order_invoice->delivery_number);
        if ($ref !== false) {
            $this->title = $ref;
        }
    }

    public function getFilename()
    {
        // MXB_DOCNUM OVERRIDE: modify delivery slip number
        $ref = MxbDocnumUtils::generateCustomReference('delivery-slip', (int)$this->order_invoice->delivery_number);
        if ($ref !== false) {
            return Configuration::get('PS_DELIVERY_PREFIX', Context::getContext()->language->id, null, $this->order->id_shop) . $ref . '.pdf';
        }
        return parent::getFilename();
    }

    protected function getTemplate($template_name)
    {
        // MXB_DOCNUM OVERRIDE: modify template folder
        $template = MxbDocnumDeliverySlipData::getTemplateOverride($template_name);
        if ($template !== false) {
            return $template;
        }
        return parent::getTemplate($template_name);
    }

    public function getHeader()
    {
        // MXB_DOCNUM OVERRIDE: add info to header
        MxbDocnumDeliverySlipData::getHeaderOverride($this->smarty, $this->order_invoice->shop_address);
        return parent::getHeader();
    }

    public function getContent()
    {
        // MXB_DOCNUM OVERRIDE: add info to body
        MxbDocnumDeliverySlipData::getContentOverride(
            $this->smarty,
            $this->order->id,
            $this->getTemplate('delivery-slip.adv-table-tab')
        );
        return parent::getContent();
    }
}
